@@ -212,6 +212,11 @@ typedef struct _st_vp {
   _st_thread_t *sleep_q;      /* sleep queue for this vp */
   int sleepq_size;	      /* number of threads on sleep queue */
 
+#ifdef ST_SWITCH_CB
+  st_switch_cb_t switch_out_cb;	/* called when a thread is switched out */
+  st_switch_cb_t switch_in_cb;	/* called when a thread is switched in */
+#endif
+
 #ifndef USE_POLL
   int maxfd;
   fd_set fd_read_set, fd_write_set, fd_exception_set;
@@ -351,8 +356,14 @@ extern _st_thread_t *_st_this_thread;
 #else
 #define ST_DEFAULT_STACK_SIZE (128*1024)  /* Includes register stack size */
 #endif
+
+#ifndef ST_KEYS_MAX
 #define ST_KEYS_MAX 16
+#endif
+
+#ifndef ST_MIN_POLLFDS_SIZE
 #define ST_MIN_POLLFDS_SIZE 64
+#endif
 
 
 /*****************************************
@@ -366,16 +377,36 @@ void _st_iterate_threads(void);
 #define ST_DEBUG_ITERATE_THREADS()
 #endif
 
+#ifdef ST_SWITCH_CB
+#define ST_SWITCH_OUT_CB(_thread)		\
+    if (_st_this_vp.switch_out_cb != NULL &&	\
+        _thread != _st_this_vp.idle_thread &&	\
+        _thread->state != _ST_ST_ZOMBIE) {	\
+      _st_this_vp.switch_out_cb();		\
+    }
+#define ST_SWITCH_IN_CB(_thread)		\
+    if (_st_this_vp.switch_in_cb != NULL &&	\
+	_thread != _st_this_vp.idle_thread &&	\
+	_thread->state != _ST_ST_ZOMBIE) {	\
+      _st_this_vp.switch_in_cb();		\
+    }
+#else
+#define ST_SWITCH_OUT_CB(_thread)
+#define ST_SWITCH_IN_CB(_thread)
+#endif
+
 /*
  * Switch away from the current thread context by saving its state and
  * calling the thread scheduler
  */
 #define _ST_SWITCH_CONTEXT(_thread)       \
     ST_BEGIN_MACRO                        \
+    ST_SWITCH_OUT_CB(_thread);            \
     if (!MD_SETJMP((_thread)->context)) { \
       _st_vp_schedule();                  \
     }                                     \
     ST_DEBUG_ITERATE_THREADS();           \
+    ST_SWITCH_IN_CB(_thread);             \
     ST_END_MACRO
 
 /*
